"""some standard constants used with an uppercut system"""
class EnvVars(object):
    """commonly used env variable names"""
    SHARED_VAR_SERVER = "SHARED_VAR_SERVER" # the env var used to store the host name of the service that manages variables shared between modules.
    MOD_REG_SERVER = "MOD_REG_SERVER" # the env var used to indicate the host name of the module registration server
    # some friendlier constant names.
    SharedVariableServer = SHARED_VAR_SERVER
    ModuleRegistrationServer = MOD_REG_SERVER

class Keys(object):
    """commonly used key names"""
    UPPERCUT_SYS = "UPPERCUT_SYS" 
    MOD_LIST = "MOD_LIST"



""" tools to configure modules used in an uppercut system """
from redis import Redis
import constants
import json
import environment
import signal
import time
from sharedvars import SharedVarStore, SystemVarStore
redishost = environment.getVar(constants.EnvVars.MOD_REG_SERVER,"localhost")
    
class Module(object):
    def terminate(self,signum,frame):
        self._isUp = False
    
    def __init__(self,name,interval):
        signal.signal(signal.SIGINT,self.terminate)
        signal.signal(signal.SIGTERM,self.terminate)
        self.VarStore = SharedVarStore(name)
        self._sysVarStore = SystemVarStore()
        self._name = name
        self._interval = interval
        self._isUp = False
        self._register()

    def _register(self):
        _redis = Redis(host=redishost)
        modInfo = {'name':self._name}
        self._sysVarStore.push(constants.Keys.MOD_LIST,modInfo)
        print('pushed mod info')

    def start(self):
        self._isUp = True
        while self._isUp:
            self.mainLoop()
            time.sleep(self._interval)

if __name__=="__main__":
    class test_mod(Module):
        def mainLoop(self):
            print(self)
    myMod = test_mod('test_mod', 1)
    myMod.start()



from redis import Redis
import json
import constants 
import environment

def encode(val):
    return json.dumps(val)

def decode(val):
    return json.loads(val.decode('utf-8'))

_redisInstance = None
def GetRedisInstance(redisHost=None):
    global _redisInstance
    if _redisInstance is None:
        redisHost = redisHost or environment.getVar(constants.EnvVars.SHARED_VAR_SERVER,'localhost')
        _redisInstance = Redis(host=redisHost)
    return _redisInstance

class SharedVarStore(Redis):
    """This class abstracts the reading and setting of shared variables between modules.
    This specific implementation uses Redis"""
    def __init__(self,moduleName,redisHost=None):
        
        self._client = GetRedisInstance(redisHost)
        self._modName = moduleName

    def getKey(self,varName):
        """generate the key name used to store the var in Redis"""
        return '{}/{}'.format(self._modName,varName)

    def get(self,varName,defaultValue=None):
        """ Get the value of a shared variable.

        @param: varName The name of the variable that you need to access.
        @param: defaultValue The default value to return if the variable doesn't exist yet.
        """
        v = self._client.get(self.getKey(varName))
        if v is None: return defaultValue
        return decode(v)

    def set(self,varName,value):
        """ Set the value of a shared variable.

        @param: varName The name of the variable to set.
        @param: value The new value of the variable.
        """
        self._client.set(self.getKey(varName),encode(value))

    def push(self,listName,value):
        k = self.getKey(listName)
        v = encode(value)
        self._client.lpush(k,v)
        self._client.lpush
    
def SystemVarStore():
    return SharedVarStore(constants.Keys.UPPERCUT_SYS)


from setuptools import setup,find_packages
from sys import argv
import json

with open('.version','r') as f:info = json.load(f)

if True or 'step' in argv:
    argv.remove('step')
    print('stepping version')
    info['version'][-1] = info['version'][-1]+1
    with open('.version','w') as f:json.dump(info,f)

print(info)

def getVersionString():
    return '.'.join([str(n) for n in info['version']])

setup(
    name='uppercut',
    url='https://github.com/StephanAm/uppercut.python.tools',
    author='Stephan Marais',
    maintainer='Stephan Marais',
    version=getVersionString(),
    description='A library to handle common tasks for creating an Uppercut module in python.',
    py_modules=['test'],
    package_dir={'':'src'},
    packages=find_packages('src'),
    classifiers=[
        "Programming Language :: Python :: 3.7"
    ],
    install_requires = [
        'redis'
    ]
)